@@ -1,3 +1,4 @@
+import datetime
 import json
 import uuid
 import sys
@@ -7,7 +8,11 @@ import requests
 import pickle
 import pandas
 
+import types
+
 import sklearn
+import wizata_dsapi
+
 from .plot import Plot
 from .request import Request
 from .mlmodel import MLModel
@@ -178,6 +183,16 @@ class WizataDSAPIClient:
             else:
                 raise self.__raise_error(response)
         if obj is not None:
+            if isinstance(obj, Execution):
+                response = requests.request("GET",
+                                            self.__url() + "execute/" + str(obj.execution_id) + "/",
+                                            headers=self.__header()
+                                            )
+                if response.status_code == 200:
+                    obj.from_json(response.json())
+                    return obj
+                else:
+                    raise self.__raise_error(response)
             if isinstance(obj, MLModel):
                 response = requests.request("GET",
                                             self.__url() + "mlmodels/" + str(obj.model_id) + "/",
@@ -246,6 +261,10 @@ class WizataDSAPIClient:
         :param obj: object to save on the server (any id is ignored and replaced)
         :return: id of created object
         """
+        if callable(obj) and isinstance(obj, types.FunctionType):
+            obj = wizata_dsapi.Script(
+                function=obj
+            )
         if isinstance(obj, Script):
             response = requests.post(self.__url() + "scripts/",
                                      headers=self.__header(),
@@ -273,6 +292,10 @@ class WizataDSAPIClient:
         :param obj: object to update on the server
         :return: None
         """
+        if callable(obj) and isinstance(obj, types.FunctionType):
+            obj = wizata_dsapi.Script(
+                function=obj
+            )
         if isinstance(obj, Script):
             response = requests.put(self.__url() + "scripts/" + str(obj.script_id) + "/",
                                     headers=self.__header(),
@@ -295,12 +318,15 @@ class WizataDSAPIClient:
     def upsert(self, obj):
         """
         upsert on object on the server
-        work with Script using the technical name of the function
+        work with Script or directly your function using the technical name of the function
 
         :param obj: object to upsert on the server
         :return: ID of the object created or updated
         """
-
+        if callable(obj) and isinstance(obj, types.FunctionType):
+            obj = wizata_dsapi.Script(
+                function=obj
+            )
         if isinstance(obj, Script):
             response = requests.put(self.__url() + "scripts/",
                                     headers=self.__header(),
@@ -357,6 +383,38 @@ class WizataDSAPIClient:
         else:
             raise TypeError("Type not supported.")
 
+    def query(self,
+              datapoints: list[str],
+              start: datetime,
+              end: datetime,
+              interval: int,
+              agg_method: str = "mean") -> pandas.DataFrame:
+        """
+        Query a dataframe from API.
+        :param agg_method:
+        :param datapoints: list of datapoints to fetch.
+        :param start: start datetime of range to fetch
+        :param end: end datetime of range to fetch
+        :param interval: interval in milliseconds.
+        :return: dataframe
+        """
+        request = wizata_dsapi.Request()
+
+        if datapoints is not None:
+            request.add_datapoints(datapoints)
+
+        if start is not None:
+            request.start = start
+
+        if end is not None:
+            request.end = end
+
+        request.set_aggregation(agg_method, interval)
+
+        return self.get(request)
+
+
+
     def execute(self,
                 execution: Execution = None,
                 request: Request = None,
